/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package application;

import globals.Constants;
import globals.Globals;
import globals.SettingsManager;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import core.GameSwitcher;

/**
 * The main menu.
 */
class ApplicationMenuBar extends JMenuBar implements ActionListener, Observer {
	GameSwitcher game_panel;

	String current_game;
	// We need access to modal buttons.
	JMenuItem open_file_button;
	JMenuItem open_defaults_button;
	JCheckBoxMenuItem grid_button;
	JMenu help_menu;
	JMenuItem about_levels_button;

	ApplicationMenuBar(GameSwitcher gs) {
		JMenuItem button;
		JMenu menu;
		JCheckBoxMenuItem checkbox;

		game_panel = gs;

		// *** FILE MENU ***
		menu = new JMenu(Globals.text.getString("menu_fileMenu"));
		menu.setMnemonic(Constants.mnemonic_fileMenu);
		menu.addActionListener(this);

		// OPEN FILE BUTTON
		open_file_button = new JMenuItem(Globals.text
				.getString("menu_openFile"));
		open_file_button.setAccelerator(KeyStroke.getKeyStroke(
				Constants.accelerator_openFile, Constants.accelerator_maskApp));
		open_file_button.setEnabled(true);
		open_file_button.addActionListener(this);
		menu.add(open_file_button);

		// OPEN DEFAULT LEVELS BUTTON
		open_defaults_button = new JMenuItem(Globals.text
				.getString("menu_openDefaults"));
		open_defaults_button.setAccelerator(KeyStroke.getKeyStroke(
				Constants.accelerator_openDefaults,
				Constants.accelerator_maskApp));
		open_defaults_button.setEnabled(true);
		open_defaults_button.addActionListener(this);
		menu.add(open_defaults_button);

		menu.addSeparator();

		// EXIT BUTTON
		button = new JMenuItem(Globals.text.getString("menu_exitApp"));
		button.setAccelerator(KeyStroke.getKeyStroke(
				Constants.accelerator_exitApp, Constants.accelerator_maskApp));
		button.addActionListener(this);
		menu.add(button);

		add(menu);

		// *** VIEW MENU ***
		menu = new JMenu(Globals.text.getString("menu_viewMenu"));
		menu.setMnemonic(Constants.mnemonic_viewMenu);

		grid_button = new JCheckBoxMenuItem(
				Globals.text.getString("menu_grid"), SettingsManager.app_grid);

		grid_button.addActionListener(this);
		grid_button.setEnabled(true);
		grid_button.setAccelerator(KeyStroke.getKeyStroke(
				Constants.accelerator_grid, Constants.accelerator_maskApp));
		menu.add(grid_button);

		add(menu);

		// *** TOOL MENU ***
		menu = new JMenu(Globals.text.getString("menu_toolMenu"));
		menu.setMnemonic(Constants.mnemonic_toolMenu);

		button = new JMenuItem(Globals.text.getString("preferences_title_menu"));
		button.setAccelerator(KeyStroke.getKeyStroke(
				Constants.accelerator_preferences,
				Constants.accelerator_maskApp));
		button.addActionListener(this);
		menu.add(button);

		button = new JMenuItem(Globals.text.getString("keySettings_title_menu"));
		button.setAccelerator(KeyStroke.getKeyStroke(
				Constants.accelerator_keySettings,
				Constants.accelerator_maskApp));
		button.addActionListener(this);
//		menu.add(button);

		checkbox = new JCheckBoxMenuItem(Globals.text
				.getString("save_preferences"));
		checkbox.setState(Globals.file_saveSettingsOnExit);
		checkbox.addActionListener(this);
		checkbox.setEnabled(true);
		menu.add(checkbox);

		add(menu);

		// *** HELP MENU ***
		help_menu = new JMenu(Globals.text.getString("menu_helpMenu"));
		help_menu.setMnemonic(Constants.mnemonic_helpMenu);

		button = new JMenuItem(Globals.text.getString("help_title_menu"));
		button.setAccelerator(KeyStroke.getKeyStroke(
				Constants.accelerator_helpApp, Constants.accelerator_maskApp));
		button.addActionListener(this);
		help_menu.add(button);

		button = new JMenuItem(Globals.text.getString("about_application_menu"));
		button.setAccelerator(KeyStroke.getKeyStroke(
				Constants.accelerator_aboutApp, Constants.accelerator_maskApp));
		button.addActionListener(this);
		help_menu.add(button);

		help_menu.addSeparator();

		// GAME HELP BUTTON
		button = new JMenuItem(Globals.text.getString("help_games_menu"));
		button
				.setAccelerator(KeyStroke.getKeyStroke(
						Constants.accelerator_helpGame,
						Constants.accelerator_maskGame));
		button.addActionListener(this);
		help_menu.add(button);

		// ABOUT LEVELS DIALOG
		about_levels_button = new JMenuItem(Globals.text
				.getString("about_levels_menu"));
		about_levels_button.setAccelerator(KeyStroke.getKeyStroke(
				Constants.accelerator_aboutLevels,
				Constants.accelerator_maskGame));
		about_levels_button.addActionListener(this);
		about_levels_button.setEnabled(false);
		help_menu.add(about_levels_button);

		add(help_menu);

		game_panel.addObserver(this);
	}

	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (command == Globals.text.getString("menu_exitApp")) {
			if ((game_panel.can_abandonWithoutQuery())
					|| (Dialogs.confirmDialog(Globals.text
							.getString("exit_button")))) {
				if (Globals.file_saveSettingsOnExit)
					SettingsManager.writeSettingsToConfigFile();
				System.exit(0);
			}
		} else if (command == Globals.text.getString("menu_openFile")) {
			if (game_panel.can_abandonWithoutQuery())
				game_panel.action_openFile();
			else if (Dialogs.confirmDialog(Globals.text
					.getString("file_open")))
				game_panel.action_openFile();
		} else if (command == Globals.text.getString("menu_openDefaults")) {
			if (game_panel.can_abandonWithoutQuery())
				game_panel.action_openDefaults();
			else if (Dialogs.confirmDialog(Globals.text
					.getString("play_defaults_button")))
				game_panel.action_openDefaults();
		} else if (command.equals(Globals.text
				.getString("keySettings_title_menu"))) {
			Dialogs.open_keySettingsDialog();
		} else if (command.equals(Globals.text
				.getString("preferences_title_menu"))) {
			Dialogs.open_preferencesDialog(game_panel);
		} else if (command == Globals.text.getString("save_preferences")) {
			Globals.file_saveSettingsOnExit = !Globals.file_saveSettingsOnExit;
		} else if (command == Globals.text.getString("menu_grid")) {
			SettingsManager.app_grid = !SettingsManager.app_grid;
			game_panel.update();
		} else if (command == Globals.text.getString("help_title_menu")) {
			Dialogs.open_helpDialog();
		} else if (command == Globals.text.getString("about_application_menu")) {
			Dialogs.applicationAbout();
		} else if (command.equals(Globals.text.getString("help_games_menu"))) {
			Dialogs.open_instructionsDialog(game_panel);
			// } else if (command.equals(Globals.text.getString("about_game")
			// + current_game + Globals.text.getString("menu_ellipsis"))) {
			// Dialogs.gameAbout(game_panel);
		} else if (command == Globals.text.getString("about_levels_menu")) {
			Dialogs
					.open_aboutLevelsDialog(game_panel
							.get_levelCollectionInfo());
		}
	}

	public void update(Observable o, Object arg) {
		about_levels_button.setEnabled(game_panel
				.can_provideLevelCollectionInfo());
	}
}
