/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package core;

import tessellation.Tessellation;

/**
 * Information about the game, e.g. Sokoban
 */
public class GameInfo {
	private String title;
	private String designer;
	private String gameYear;
	private String implementor;
	private String implementationYear;
	private String email;
	private String[] description;
	private Tessellation tessellation;
	
	/**
	 * Returns the title of the game
	 */
	public GameInfo (String t, String d, String gy, String i, String iy, String e, String[] da, Tessellation ts) {
		title = t;
		designer = d;
		gameYear = gy;
		implementor = i;
		implementationYear = iy;
		email = e;
		description = da;
		tessellation = ts;
	}

	public String get_title() {
		return title;
	}
	
	/**
	 * the original game designer
	 */
	public String get_originalGameDesigner() {
		return designer;
	}

	/**
	 * the copyright year of the game
	 */
	public String get_originalGameYear() {
		return gameYear;
	}

	/**
	 * the implementer of this instance of the game
	 */
	public String get_implementor() {
		return implementor;
	}

	/**
	 * the copyright year of this implementation of the game
	 */
	public String get_implementationCopyrightYear() {
		return implementationYear;
	}

	/**
	 * The email address of the implementor
	 */
	public String get_implementorEMail() {
		return email;
	}

	/**
	 * Should return description of the game.
	 * 
	 * e.g. basic instructions.
	 */
	public String[] get_gameDescription() {
		return description;
	}
	
	public Tessellation get_tessellation() {
		return tessellation;
	}
}