/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package core;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

/**
 * The interface which GameManagers implement.
 */
public interface GameManagerInterface {
	/*
	 * ACCESSORS
	 */
	/**
	 * Get the text to display in the infobar
	 */
	String[] get_infoText();

	/**
	 * Get the text to display in the about levels box
	 */
	LevelCollectionInfo get_levelCollectionInfo();

	/*
	 * ACTIONS
	 */
	/**
	 * Go to the given level, if valid.
	 */
	void action_go(int index);

	/**
	 * Move to the previous level if there is one, otherwise ignore
	 */
	void action_goPrevious();

	/**
	 * Called when the user has indicated that they want to retry the current
	 * level, usually by pressing a button.
	 */
	void action_retry();

	/**
	 * Move to the next level if there is one, otherwise ignore
	 */
	void action_goNext();

	/**
	 * Does the game support an undo, and if so, is there something to undo
	 */
	void action_undo();

	/**
	 * Called when the user clicks on the GamePanel.
	 */
	void action_mouse(MouseEvent e);

	/**
	 * Called when the user presses a key.
	 */
	void action_key(KeyEvent e);

	/**
	 * Called when the user indicates that they want to procede (eg dismisses
	 * the Game Over screen)
	 */
	void action_okay();

	/*
	 * QUERIES
	 */

	/**
	 * Can the game move to the previous level now.
	 */
	boolean can_goPrevious();

	/**
	 * Can the game permit a retry action now.
	 */
	boolean can_retry();

	/**
	 * Can the game move to the next level now.
	 */
	boolean can_goNext();

	/**
	 * Can the game permit an undo action now.
	 */
	boolean can_undo();

	/**
	 * Can we exit the game without a query now
	 */
	boolean can_abandonWithoutQuery();

	/**
	 * Can we provide info about the levels
	 */
	boolean can_provideLevelCollectionInfo();
	
	/*
	 * Painting code
	 */
	void paintPanel(Graphics2D g2, Insets i, Dimension d);
	
}
