/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package globals;

import file.FileManager;

import java.awt.event.KeyEvent;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import tessellation.TessellationManager;
import core.AvailableGameManager;

/**
 * Manages the settings for the application which may get changed.
 * 
 * They are obtained from the defaults and then can be overridden by either the
 * Applet context or a configuration file
 */
public class SettingsManager {
	/**
	 * The amount by which one of a unit's sides must be kept proportional to
	 * the other - some deformation is allowed
	 */
	public static double graphics_Xproportion;
	public static double graphics_Yproportion;

	/**
	 * Should graphics be antialiased
	 */
	public static boolean graphics_antialiasing;

	/**
	 * When the user is playing a game and performs an action that will cause
	 * that game to be lost, should we ask for confirmation?
	 */
	public static boolean app_queryForQuits;

	public static boolean app_grid;
	
	/**
	 * The number of allowed undos (unbounded if negative).
	 */
	public static int app_maxUndos;
	
	/**
	 * Return general settings to their original values.
	 */
	public static void restoreSettings() {
		//FIXME this needs to be a lot more defensive.
		graphics_Xproportion = Double.parseDouble(Defaults.graphics_Xproportion);
		graphics_Yproportion = Double.parseDouble(Defaults.graphics_Yproportion);
		graphics_antialiasing = Defaults.graphics_antialiasing.equals("true");
		app_queryForQuits = Defaults.app_queryForQuits.equals("true");
		app_grid = Defaults.app_grid.equals("true");
		app_maxUndos = Integer.parseInt(Defaults.app_maxUndos);
		// Get any game specific settings
		AvailableGameManager.restoreSettings();
	}

	/**
	 * Restore key settings to their original values.
	 */
	public static void restoreKeySettings() {
		TessellationManager.restoreDefaults();
	}

	public static void getConfigFileSettings() {
		try {
			Globals.settings.load(new FileInputStream(Globals.file_configFile));
		} catch (IOException e) {
			// Do nothing, the defaults will be fine
		}
		accessSettings();
		accessKeySettings();
	}

	public static final KeyCodeAssignment[] keyCodes = {
			new KeyCodeAssignment("0", KeyEvent.VK_0),
			new KeyCodeAssignment("1", KeyEvent.VK_1),
			new KeyCodeAssignment("2", KeyEvent.VK_2),
			new KeyCodeAssignment("3", KeyEvent.VK_3),
			new KeyCodeAssignment("4", KeyEvent.VK_4),
			new KeyCodeAssignment("5", KeyEvent.VK_5),
			new KeyCodeAssignment("6", KeyEvent.VK_6),
			new KeyCodeAssignment("7", KeyEvent.VK_7),
			new KeyCodeAssignment("8", KeyEvent.VK_8),
			new KeyCodeAssignment("9", KeyEvent.VK_9),
			new KeyCodeAssignment("A", KeyEvent.VK_A),
			new KeyCodeAssignment("ADD", KeyEvent.VK_ADD),
			new KeyCodeAssignment("ALT", KeyEvent.VK_ALT),
			new KeyCodeAssignment("ALT_GRAPH", KeyEvent.VK_ALT_GRAPH),
			new KeyCodeAssignment("AMPERSAND", KeyEvent.VK_AMPERSAND),
			new KeyCodeAssignment("ASTERISK", KeyEvent.VK_ASTERISK),
			new KeyCodeAssignment("AT", KeyEvent.VK_AT),
			new KeyCodeAssignment("B", KeyEvent.VK_B),
			new KeyCodeAssignment("BACK_QUOTE", KeyEvent.VK_BACK_QUOTE),
			new KeyCodeAssignment("BACK_SLASH", KeyEvent.VK_BACK_SLASH),
			new KeyCodeAssignment("BACK_SPACE", KeyEvent.VK_BACK_SPACE),
			new KeyCodeAssignment("BRACELEFT", KeyEvent.VK_BRACELEFT),
			new KeyCodeAssignment("BRACERIGHT", KeyEvent.VK_BRACERIGHT),
			new KeyCodeAssignment("C", KeyEvent.VK_C),
			new KeyCodeAssignment("CAPS_LOCK", KeyEvent.VK_CAPS_LOCK),
			new KeyCodeAssignment("CIRCUMFLEX", KeyEvent.VK_CIRCUMFLEX),
			new KeyCodeAssignment("CLOSE_BRACKET", KeyEvent.VK_CLOSE_BRACKET),
			new KeyCodeAssignment("COLON", KeyEvent.VK_COLON),
			new KeyCodeAssignment("COMMA", KeyEvent.VK_COMMA),
			new KeyCodeAssignment("D", KeyEvent.VK_D),
			new KeyCodeAssignment("DELETE", KeyEvent.VK_DELETE),
			new KeyCodeAssignment("DIVIDE", KeyEvent.VK_DIVIDE),
			new KeyCodeAssignment("DOLLAR", KeyEvent.VK_DOLLAR),
			new KeyCodeAssignment("DOWN", KeyEvent.VK_DOWN),
			new KeyCodeAssignment("E", KeyEvent.VK_E),
			new KeyCodeAssignment("END", KeyEvent.VK_END),
			new KeyCodeAssignment("ENTER", KeyEvent.VK_ENTER),
			new KeyCodeAssignment("EQUALS", KeyEvent.VK_EQUALS),
			new KeyCodeAssignment("ESCAPE", KeyEvent.VK_ESCAPE),
			new KeyCodeAssignment("EURO_SIGN", KeyEvent.VK_EURO_SIGN),
			new KeyCodeAssignment("EXCLAMATION_MARK",
					KeyEvent.VK_EXCLAMATION_MARK),
			new KeyCodeAssignment("F", KeyEvent.VK_F),
			new KeyCodeAssignment("F1", KeyEvent.VK_F1),
			new KeyCodeAssignment("F10", KeyEvent.VK_F10),
			new KeyCodeAssignment("F11", KeyEvent.VK_F11),
			new KeyCodeAssignment("F12", KeyEvent.VK_F12),
			new KeyCodeAssignment("F13", KeyEvent.VK_F13),
			new KeyCodeAssignment("F14", KeyEvent.VK_F14),
			new KeyCodeAssignment("F15", KeyEvent.VK_F15),
			new KeyCodeAssignment("F16", KeyEvent.VK_F16),
			new KeyCodeAssignment("F17", KeyEvent.VK_F17),
			new KeyCodeAssignment("F18", KeyEvent.VK_F18),
			new KeyCodeAssignment("F19", KeyEvent.VK_F19),
			new KeyCodeAssignment("F2", KeyEvent.VK_F2),
			new KeyCodeAssignment("F20", KeyEvent.VK_F20),
			new KeyCodeAssignment("F21", KeyEvent.VK_F21),
			new KeyCodeAssignment("F22", KeyEvent.VK_F22),
			new KeyCodeAssignment("F23", KeyEvent.VK_F23),
			new KeyCodeAssignment("F24", KeyEvent.VK_F24),
			new KeyCodeAssignment("F3", KeyEvent.VK_F3),
			new KeyCodeAssignment("F4", KeyEvent.VK_F4),
			new KeyCodeAssignment("F5", KeyEvent.VK_F5),
			new KeyCodeAssignment("F6", KeyEvent.VK_F6),
			new KeyCodeAssignment("F7", KeyEvent.VK_F7),
			new KeyCodeAssignment("F8", KeyEvent.VK_F8),
			new KeyCodeAssignment("F9", KeyEvent.VK_F9),
			new KeyCodeAssignment("G", KeyEvent.VK_G),
			new KeyCodeAssignment("GREATER", KeyEvent.VK_GREATER),
			new KeyCodeAssignment("H", KeyEvent.VK_H),
			new KeyCodeAssignment("HOME", KeyEvent.VK_HOME),
			new KeyCodeAssignment("I", KeyEvent.VK_I),
			new KeyCodeAssignment("INSERT", KeyEvent.VK_INSERT),
			new KeyCodeAssignment("INVERTED_EXCLAMATION_MARK",
					KeyEvent.VK_INVERTED_EXCLAMATION_MARK),
			new KeyCodeAssignment("J", KeyEvent.VK_J),
			new KeyCodeAssignment("K", KeyEvent.VK_K),
			new KeyCodeAssignment("KP_DOWN", KeyEvent.VK_KP_DOWN),
			new KeyCodeAssignment("KP_LEFT", KeyEvent.VK_KP_LEFT),
			new KeyCodeAssignment("KP_RIGHT", KeyEvent.VK_KP_RIGHT),
			new KeyCodeAssignment("KP_UP", KeyEvent.VK_KP_UP),
			new KeyCodeAssignment("L", KeyEvent.VK_L),
			new KeyCodeAssignment("LEFT", KeyEvent.VK_LEFT),
			new KeyCodeAssignment("LEFT_PARENTHESIS",
					KeyEvent.VK_LEFT_PARENTHESIS),
			new KeyCodeAssignment("LESS", KeyEvent.VK_LESS),
			new KeyCodeAssignment("M", KeyEvent.VK_M),
			new KeyCodeAssignment("META", KeyEvent.VK_META),
			new KeyCodeAssignment("MINUS", KeyEvent.VK_MINUS),
			new KeyCodeAssignment("MULTIPLY", KeyEvent.VK_MULTIPLY),
			new KeyCodeAssignment("N", KeyEvent.VK_N),
			new KeyCodeAssignment("NUM_LOCK", KeyEvent.VK_NUM_LOCK),
			new KeyCodeAssignment("NUMBER_SIGN", KeyEvent.VK_NUMBER_SIGN),
			new KeyCodeAssignment("NUMPAD0", KeyEvent.VK_NUMPAD0),
			new KeyCodeAssignment("NUMPAD1", KeyEvent.VK_NUMPAD1),
			new KeyCodeAssignment("NUMPAD2", KeyEvent.VK_NUMPAD2),
			new KeyCodeAssignment("NUMPAD3", KeyEvent.VK_NUMPAD3),
			new KeyCodeAssignment("NUMPAD4", KeyEvent.VK_NUMPAD4),
			new KeyCodeAssignment("NUMPAD5", KeyEvent.VK_NUMPAD5),
			new KeyCodeAssignment("NUMPAD6", KeyEvent.VK_NUMPAD6),
			new KeyCodeAssignment("NUMPAD7", KeyEvent.VK_NUMPAD7),
			new KeyCodeAssignment("NUMPAD8", KeyEvent.VK_NUMPAD8),
			new KeyCodeAssignment("NUMPAD9", KeyEvent.VK_NUMPAD9),
			new KeyCodeAssignment("O", KeyEvent.VK_O),
			new KeyCodeAssignment("OPEN_BRACKET", KeyEvent.VK_OPEN_BRACKET),
			new KeyCodeAssignment("P", KeyEvent.VK_P),
			new KeyCodeAssignment("PAGE_DOWN", KeyEvent.VK_PAGE_DOWN),
			new KeyCodeAssignment("PAGE_UP", KeyEvent.VK_PAGE_UP),
			new KeyCodeAssignment("PAUSE", KeyEvent.VK_PAUSE),
			new KeyCodeAssignment("PERIOD", KeyEvent.VK_PERIOD),
			new KeyCodeAssignment("PLUS", KeyEvent.VK_PLUS),
			new KeyCodeAssignment("PRINTSCREEN", KeyEvent.VK_PRINTSCREEN),
			new KeyCodeAssignment("Q", KeyEvent.VK_Q),
			new KeyCodeAssignment("QUOTE", KeyEvent.VK_QUOTE),
			new KeyCodeAssignment("QUOTEDBL", KeyEvent.VK_QUOTEDBL),
			new KeyCodeAssignment("R", KeyEvent.VK_R),
			new KeyCodeAssignment("RIGHT", KeyEvent.VK_RIGHT),
			new KeyCodeAssignment("RIGHT_PARENTHESIS",
					KeyEvent.VK_RIGHT_PARENTHESIS),
			new KeyCodeAssignment("ROMAN_CHARACTERS",
					KeyEvent.VK_ROMAN_CHARACTERS),
			new KeyCodeAssignment("S", KeyEvent.VK_S),
			new KeyCodeAssignment("SCROLL_LOCK", KeyEvent.VK_SCROLL_LOCK),
			new KeyCodeAssignment("SEMICOLON", KeyEvent.VK_SEMICOLON),
			new KeyCodeAssignment("SEPARATER", KeyEvent.VK_SEPARATER),
			new KeyCodeAssignment("SEPARATOR", KeyEvent.VK_SEPARATOR),
			new KeyCodeAssignment("SHIFT", KeyEvent.VK_SHIFT),
			new KeyCodeAssignment("SLASH", KeyEvent.VK_SLASH),
			new KeyCodeAssignment("SPACE", KeyEvent.VK_SPACE),
			new KeyCodeAssignment("SUBTRACT", KeyEvent.VK_SUBTRACT),
			new KeyCodeAssignment("T", KeyEvent.VK_T),
			new KeyCodeAssignment("TAB", KeyEvent.VK_TAB),
			new KeyCodeAssignment("U", KeyEvent.VK_U),
			new KeyCodeAssignment("UNDERSCORE", KeyEvent.VK_UNDERSCORE),
			new KeyCodeAssignment("UP", KeyEvent.VK_UP),
			new KeyCodeAssignment("V", KeyEvent.VK_V),
			new KeyCodeAssignment("W", KeyEvent.VK_W),
			new KeyCodeAssignment("X", KeyEvent.VK_X),
			new KeyCodeAssignment("Y", KeyEvent.VK_Y),
			new KeyCodeAssignment("Z", KeyEvent.VK_Z) };

	/**
	 * Look up a keycode using a key description. If we don't find the
	 * corresponding keycode, return the provided default.
	 */
	public static int parseKeyDescription(String desc, int defaultCode) {
		int i;
		for (i = 0; i < keyCodes.length; i++)
			if (keyCodes[i].description.equals(desc.trim().toUpperCase()))
				return keyCodes[i].keyCode;
		return defaultCode;
	}

	public static String[] getAcceptableKeys() {
		String[] sa = new String[keyCodes.length];
		for (int i = 0; i < keyCodes.length; i++)
			sa[i] = keyCodes[i].description;
		return sa;
	}

	/**
	 */
	public static void accessSettings() {
		//FIXME this needs to be a lot more defensive.
		graphics_Xproportion = Double.parseDouble(Globals.settings.getProperty("graphics_Xproportion",Defaults.graphics_Xproportion));
		graphics_Yproportion = Double.parseDouble(Globals.settings.getProperty("graphics_Yproportion",Defaults.graphics_Yproportion));
		graphics_antialiasing = Globals.settings.getProperty("graphics_antialiasing",Defaults.graphics_antialiasing).equals("true");
		app_queryForQuits = Globals.settings.getProperty("app_queryForQuits",Defaults.app_queryForQuits).equals("true");
		app_grid = Globals.settings.getProperty("app_grid",Defaults.app_grid).equals("true");
		app_maxUndos = Integer.parseInt(Globals.settings.getProperty("app_undoLevel",Defaults.app_maxUndos));
		// Get any game specific settings
		AvailableGameManager.accessSettings();
		// Get any file specific settings
		FileManager.accessSettings();
	}
	
	public static void accessKeySettings() {
		// Get any tessellation specific settings
		TessellationManager.accessKeySettings();	
	}

	public static void writeSettingsToConfigFile() {
		updateSettings();
		updateKeySettings();
		try {
			Globals.settings.store(new FileOutputStream(Globals.file_configFile),"");
		} catch (FileNotFoundException e) {
			System.out.println("Unable to write configuration file0" + e);
			System.out.println(Globals.file_configFile);
		} catch (IOException e) {
			System.out.println("Unable to write configuration file1" + e);
			System.out.println(Globals.file_configFile);
		}
	}

	public static void updateSettings() {
		Globals.settings.setProperty("graphics_Xproportion",Double.toString(graphics_Xproportion));
		Globals.settings.setProperty("graphics_Yproportion",Double.toString(graphics_Yproportion));
		Globals.settings.setProperty("graphics_antialiasing",Boolean.toString(graphics_antialiasing));
		Globals.settings.setProperty("app_queryForQuits",Boolean.toString(app_queryForQuits));
		Globals.settings.setProperty("app_grid",Boolean.toString(app_grid));
		Globals.settings.setProperty("app_undoLevel", Integer.toString(app_maxUndos));
		FileManager.updateSettings();
		// Get any game specific settings
		AvailableGameManager.updateSettings();
	}

	public static void updateKeySettings() {
		// Get any tessellation specific settings
		TessellationManager.updateKeySettings();	
	}
}