/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package level;


import java.util.*;

import tessellation.Direction;
import tessellation.Position;

/**
 * An abstract class which provides an interface for general shapes
 */
public abstract class UnitShape extends UnitObject {
	UnitShape() {
		// FIXME
		selector = unselected;
		selectable = new boolean[2];
		selectable[0] = false;
		selectable[1] = false;
	}

	static final int unselected = -1;

	boolean isSelected() {
		return selector > unselected;
	}

	boolean isSelected(int superSelection) {
		return applySuperSelection(superSelection) > unselected;
	}

	int applySuperSelection(int superSelection) {
		return StrictMath.max(selector, superSelection);
	}

	/**
	 * returns those positions which the object would newly cover, if it is
	 * currently at pos and it were moved in direction dir.
	 * 
	 * The reason we require a position (as opposed to doing a relative
	 * calculation later) is that in some tessellations "neighbour" varies
	 * depending on actual position
	 */
	LinkedList<Position> neighbours(Position pos, Direction dir) {
		LinkedList<Position> pl = new LinkedList<Position>();
		Iterator<Position> pi;
		Position pos1;
		for (pi = nonEmptyPositions().iterator(); pi.hasNext();) {
			pos1 = pos.plus(pi.next()).neighbour(dir);
			if (empty(pos1))
				pl.add(pos1);
		}
		return pl;
	}

	/**
	 * Returns the resistance to moving this shape in the given direction By
	 * default it returns no resistance
	 */
	protected double resistance(Direction dir) {
		return Resistance.none;
	}

	/**
	 * Is this shape is selectable by player[x]?
	 */
	boolean[] selectable;

	/**
	 * Who is selecting it now?
	 */
	int selector;
}
