/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package level;

/* UnitShapeBounds.java -
 *
 * Copyright (C) 2002-2005 Malcolm Tyrrell
 * malcohol@eircom.net
 *
 * This code may be used and distributed under the terms of the GNU General
 * Public Licence.
 */

import java.awt.geom.*;

import tessellation.Direction;
import tessellation.Position;

/**
 * A class for holding the bounds of a UnitShape, used for judging how much
 * screen area it will use up.
 */
class UnitShapeBounds {
	/*
	 * The reason these hold Positions and not Point2D's, is that I need to be
	 * able to make the bounds relative to a Position.
	 */
	private Position top;
	private Position right;
	private Position bottom;
	private Position left;

	/**
	 * Initializes the object with all the bounds set to the given position
	 */
	UnitShapeBounds(Position p) {
		top = p;
		right = p;
		bottom = p;
		left = p;
	}

	/**
	 * Initializes the object with all the bounds set to the given bounds plus
	 * the offset
	 */
	UnitShapeBounds(UnitShapeBounds other, Position pos) {
		top = pos.plus(other.top);
		right = pos.plus(other.right);
		bottom = pos.plus(other.bottom);
		left = pos.plus(other.left);
	}

	/**
	 * Update the bounds to include the provided position
	 */
	void updateBounds(Position pos) {
		top = top.furthestUp(pos);
		right = right.furthestRight(pos);
		bottom = bottom.furthestDown(pos);
		left = left.furthestLeft(pos);
	}

	/**
	 * Update the bounds to include the provided bounds
	 */
	void updateBounds(UnitShapeBounds other) {
		top = top.furthestUp(other.top);
		right = right.furthestRight(other.right);
		bottom = bottom.furthestDown(other.bottom);
		left = left.furthestLeft(other.left);
	}

	/**
	 * Update the bounds to include the provided bounds at the given offset
	 */
	void updateBoundsOffset(UnitShapeBounds other, Position pos) {
		top = top.furthestUp(pos.plus(other.top));
		right = right.furthestRight(pos.plus(other.right));
		bottom = bottom.furthestDown(pos.plus(other.bottom));
		left = left.furthestLeft(pos.plus(other.left));
	}

	/**
	 * Extend the bounds in the given direction
	 */
	void extendBounds(Direction dir) {
		top = top.furthestUp(top.neighbour(dir));
		right = right.furthestRight(right.neighbour(dir));
		bottom = bottom.furthestDown(bottom.neighbour(dir));
		left = left.furthestLeft(left.neighbour(dir));
	}

	/**
	 * returns the coordinates of the upperlefthand corner which bounds the
	 * given position
	 */
	Point2D topLeftCornerPoint() {
		return new Point2D.Double(left.topLeftCornerPoint().getX(), top
				.topLeftCornerPoint().getY());
	}

	/**
	 * returns the coordinates of the lowerrighthand corner which bounds the
	 * given position
	 */
	Point2D bottomRightCornerPoint() {
		return new Point2D.Double(right.bottomRightCornerPoint().getX(), bottom
				.bottomRightCornerPoint().getY());
	}

	public String toString() {
		return (top.toString() + right + bottom + left);
	}
}
