/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package tessellation;

import globals.Globals;

import java.awt.event.KeyEvent;
import java.awt.geom.Point2D;

/**
 * A Tessellation Factory for the Square Tessellation.
 */
class SquareFactory extends TessellationFactory {
	SquareFactory() {
		tessellation_prefix = "square";
		tessellation_name = "Squares";
		number_of_directions = 4;
		String[] ds = { Globals.text.getString("north"),
				Globals.text.getString("east"),
				Globals.text.getString("south"), Globals.text.getString("west") };
		direction_names = ds;
		String[] dsfx = { "north", "east", "south", "west" };
		direction_suffixes = dsfx;
		int[] ks = { KeyEvent.VK_UP, KeyEvent.VK_RIGHT, KeyEvent.VK_DOWN,
				KeyEvent.VK_LEFT };
		key_defaults = ks;
		key_settings = new int[4];
	}

	public Direction newDirection() {
		return new SquareDirection();
	}

	public Direction newDirection(int startDir) {
		return new SquareDirection(startDir);
	}

	public Position newPosition() {
		return new SquarePosition();
	}

	public Position newPosition(int array_x, int array_y) {
		return new SquarePosition(array_x, array_y);
	}

	public Position newPosition(Position pos) {
		return new SquarePosition(pos);
	}
	
	public Position newPosition(Point2D p) {
		return new SquarePosition(p);
	}
}
