/* The MazezaM Engine 
 * Copyright (C) 2002-2008 Malcolm Tyrrell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package tessellation;

import globals.Globals;
import globals.Settings;
import globals.SettingsManager;

import java.awt.event.KeyEvent;
import java.awt.geom.Point2D;

import application.KeySettingsDialog;
import application.KeySettingsPanel;

import core.AdvanceSelectionPlayerAction;
import core.DirectionPlayerAction;
import core.PlayerAction;

/**
 * A class used for creating positions and directions.
 */
public abstract class TessellationFactory {
	protected String tessellation_prefix;
	protected String tessellation_name;
	protected int number_of_directions;
	protected String[] direction_names;
	protected String[] direction_suffixes;
	protected int[] key_defaults;
	protected int[] key_settings;

	/**
	 * Given a key, return the player and direction it represents (or neither)
	 */
	PlayerAction getPlayerDirectionFromKey(int k) {
		for (int j = 0; j < number_of_directions; j++) {
			if (k == key_settings[j])
				return new DirectionPlayerAction(0, newDirection(j));
		}
		return new AdvanceSelectionPlayerAction(0);
	}

	/**
	 * Create a keySettingPanel
	 */
	KeySettingsPanel newKeySettingsPanel(KeySettingsDialog ksd) {
		return new KeySettingsPanel(ksd, tessellation_name, direction_names,
				key_settings);
	}

	/**
	 * Restore the default values for this factory
	 */
	void restoreDefaults() {
		for (int j = 0; j < number_of_directions; j++)
			key_settings[j] = key_defaults[j];
	}

	/**
	 * update any settings
	 */
	void accessKeySettings() {
		String temp;
		for (int j = 0; j < number_of_directions; j++) {
			temp = Globals.settings.getProperty(tessellation_prefix
					+ "_keys_" + direction_suffixes[j]);
			if (temp != null)
				key_settings[j] = SettingsManager.parseKeyDescription(temp,
						key_defaults[j]);
		}
	}

	/**
	 * update any settings from the setting callback object
	 */
	void getSettings(Settings scb) {
		String temp;
		for (int j = 0; j < number_of_directions; j++) {
			temp = scb.getProperty(tessellation_prefix + "_keys_" + direction_suffixes[j]);
			if (temp != null)
				key_settings[j] = SettingsManager.parseKeyDescription(temp,
						key_defaults[j]);
		}
	}

	void updateKeySettings() {
		for (int j = 0; j < number_of_directions; j++)
			Globals.settings.setProperty(tessellation_prefix + "_keys_"
					+ direction_suffixes[j], KeyEvent
					.getKeyText(key_settings[j]));
	}
	
	/*
	 * FACTORY METHODS
	 * 
	 * subclasses must override these
	 */
	public abstract Direction newDirection();

	public abstract Direction newDirection(int startDir);

	public abstract Position newPosition();

	public abstract Position newPosition(int array_x, int array_y);

	public abstract Position newPosition(Position pos);
	
	/**
	 * Return the position containing the provided point.
	 */
	public abstract Position newPosition(Point2D p);
}